#include "stdafx.h"
#include "IniUtil.h"

#define	INIUTIL_DELIM_CR	L'\r'
#define	INIUTIL_DELIM_LF	L'\n'
#define	INIUTIL_DELIM_BRA	L'['
#define	INIUTIL_DELIM_KET	L']'
#define	INIUTIL_DELIM_EQ	L'='

static
LPWSTR
GetOneLine(
	LPWSTR	pFileBuf,	// IN
	LPDWORD	pdwSize		// OUT
)
{
	LPWSTR	pNextPos = NULL;

	pNextPos = wcschr(pFileBuf, INIUTIL_DELIM_CR);
	if (pNextPos != NULL) {

		*pdwSize = pNextPos - pFileBuf;

		pNextPos++;
		if (*pNextPos == INIUTIL_DELIM_LF) {
			pNextPos++;
		}
	}

	return pNextPos;
}


static
BOOL
IsSectionMatch(
	LPWSTR	pLine,
	DWORD	dwSize,
	LPWSTR	pSection
)
{
	BOOL	bRet;
	LPWSTR	p;
	DWORD	dwSectionLen;

	bRet = FALSE;

	dwSectionLen = wcslen(pSection);

	if (dwSize == dwSectionLen + 2) {
		p = pLine;

		if (*p == INIUTIL_DELIM_BRA) {
			p++;

			if (wcsnicmp(p, pSection, dwSectionLen) == 0) {
				p += dwSectionLen;

				if (*p == INIUTIL_DELIM_KET) {
					bRet = TRUE;
				}
			}
		}
	}

	return bRet;
}

static
BOOL
IsSectionLine(
	LPWSTR	pLine,
	DWORD	dwSize
)
{
	BOOL	bRet;
	LPWSTR	p;
	DWORD	i;
	DWORD	dwRestSize;

	bRet = FALSE;

	if (dwSize > 2) {
		p = pLine;
		dwRestSize = dwSize;

		if (*p == INIUTIL_DELIM_BRA) {
			p++;
			dwRestSize--;

			for (i = 0; i < dwRestSize; i++) {
				if (*p == INIUTIL_DELIM_KET) {
					bRet = TRUE;
					break;
				}
				p++;
			}
		}
	}

	return bRet;
}

static
LPWSTR
FindSection(
	LPWSTR	pFileBuf,
	LPWSTR	pSection
)
{
	LPWSTR	pLine		= NULL;
	LPWSTR	pNextLine	= NULL;
	LPWSTR	pRet		= NULL;
	DWORD	dwSize;

	pLine = pFileBuf;

	while (pLine != NULL) {
		pNextLine = GetOneLine(pLine, &dwSize);
		if (IsSectionMatch(pLine, dwSize, pSection)) {
			pRet = pNextLine;
			break;
		}

		pLine = pNextLine;
	}

	return pRet;
}

static
LPWSTR
FindEntry(
	LPWSTR	pFileBuf,
	LPWSTR	pKey,
	LPWSTR*	ppValue,
	LPDWORD	pdwValueLen
)
{
	LPWSTR	pLine		= NULL;
	LPWSTR	pNextLine	= NULL;
	LPWSTR	pRet		= NULL;
	DWORD	dwSize;
	DWORD	dwKeyLen;

	dwKeyLen = wcslen(pKey);
	pLine = pFileBuf;

	while (pLine != NULL) {
		pNextLine = GetOneLine(pLine, &dwSize);
		if (IsSectionLine(pLine, dwSize)) {
			break;
		}
		else {
			if (dwSize > dwKeyLen) {
				LPWSTR	p;
				DWORD	dwRestSize;

				p = pLine;
				dwRestSize = dwSize;

				if(wcsnicmp(p, pKey, dwKeyLen) == 0) {
					p += dwKeyLen;
					dwRestSize -= dwKeyLen;

					if (dwRestSize > 0) {
						if (*p == INIUTIL_DELIM_EQ) {
							p++;
							dwRestSize--;

							*ppValue = p;
							*pdwValueLen = dwRestSize;
							pRet = pNextLine;

							break;
						}
					}
				}

			}
		}

		pLine = pNextLine;
	}

	return pRet;
}

BOOL
GetIniString(
	LPWSTR	pAppName,			// IN		Section
	LPWSTR	pKeyName,			// IN		Key
	LPWSTR	pReturnedString,	// OUT		擾
	LPDWORD	pdwSize,			// IN, OUT	obt@̕(NULL܂)
	LPWSTR	pFileBuf			// IN		Init@C̃f[^
)
{
	BOOL	bRet;
	LPWSTR	pBuf		= NULL;
	LPWSTR	pValue		= NULL;
	DWORD	dwValueLen	= 0;

	bRet = FALSE;

	if (pAppName == NULL || pKeyName == NULL || pdwSize == NULL) {
		return FALSE;
	}

	pBuf = FindSection(pFileBuf, pAppName);
	if (pBuf != NULL) {
		if (FindEntry(pBuf, pKeyName, &pValue, &dwValueLen) != NULL) {
			if (pReturnedString != NULL) {
				if (*pdwSize > dwValueLen) {
					memcpy(pReturnedString, pValue, dwValueLen*sizeof(WCHAR));
					pReturnedString[dwValueLen] = L'\0';

					bRet = TRUE;
				}
			}
			else {
				bRet = TRUE;
			}

			*pdwSize = dwValueLen + 1;
		}
	}

	return bRet;
	
}
